@@ -1,5 +1,7 @@
+import wxee.xarray
 from wxee.collection import ImageCollection
 from wxee.image import Image
 from wxee.time_series import TimeSeries
+from wxee.utils import Initialize
 
-__version__ = "0.2.0"
+__version__ = "0.3.2"
