@@ -1,11 +1,14 @@
 from typing import Any, List, Optional, Union
 
 import ee  # type: ignore
+import pandas as pd  # type: ignore
+import plotly.express as px  # type: ignore
+import plotly.graph_objects as go  # type: ignore
 
 from wxee.climatology import Climatology, ClimatologyFrequencyEnum
 from wxee.interpolation import InterpolationMethodEnum
 from wxee.params import ParamEnum
-from wxee.utils import _normalize
+from wxee.utils import _millis_to_datetime, _normalize
 
 
 class TimeFrequencyEnum(ParamEnum):
@@ -19,6 +22,14 @@ class TimeFrequencyEnum(ParamEnum):
     minute = "minute"
 
 
+class WindowAlignEnum(ParamEnum):
+    """Parameters defining rolling window alignment options."""
+
+    left = "left"
+    center = "center"
+    right = "right"
+
+
 class TimeSeries(ee.imagecollection.ImageCollection):
     """An image collection of chronological images."""
 
@@ -72,9 +83,11 @@ class TimeSeries(ee.imagecollection.ImageCollection):
         >>> imgs.interval("day").getInfo()
         5.03
         """
-        return self.end_time.difference(self.start_time, unit=unit).divide(self.size())
+        return self.end_time.difference(self.start_time, unit=unit).divide(
+            self.size().subtract(1)
+        )
 
-    def describe(self, unit: str = "day") -> None:
+    def describe(self, unit: str = "day") -> None:  # pragma: no cover
         """Generate and print descriptive statistics about the Time Series such as the ID, start and end dates, and time between images.
         This requires pulling data from the server, so it may run slowly.
 
@@ -97,11 +110,7 @@ class TimeSeries(ee.imagecollection.ImageCollection):
         start = start_millis.format("yyyy-MM-dd HH:mm:ss z").getInfo()
         end = end_millis.format("yyyy-MM-dd HH:mm:ss z").getInfo()
 
-        # Calculating the interval using the start and end we already pulled is slightly faster than calculating the interval
-        # from scratch.
-        mean_interval = (
-            end_millis.difference(start_millis, unit=unit).divide(size).getInfo()
-        )
+        mean_interval = self.interval().getInfo()
 
         id = self.get("system:id").getInfo()
 
@@ -113,6 +122,87 @@ class TimeSeries(ee.imagecollection.ImageCollection):
             f"\n\tMean interval: {mean_interval:.2f} {unit}s"
         )
 
+    def dataframe(self) -> pd.DataFrame:
+        """Generate a Pandas dataframe describing the system properties of each image in the time series.
+
+        Returns
+        -------
+        pd.DataFrame
+            A Pandas dataframe where each row represents an image and columns represent system properties.
+        """
+        starts_millis = self.aggregate_array("system:time_start").getInfo()
+        ends_millis = self.aggregate_array("system:time_end").getInfo()
+        ids = self.aggregate_array("system:id").getInfo()
+        collection_id = self.get("system:id").getInfo()
+
+        starts = [_millis_to_datetime(ms) for ms in starts_millis]
+        ends = [_millis_to_datetime(ms) for ms in ends_millis]
+
+        df = pd.DataFrame({"id": ids, "time_start": starts, "time_end": ends})
+        df.index.id = collection_id
+        return df
+
+    def timeline(self) -> go.Figure:  # pragma: no cover
+        """Generate an interactive plot showing the acquisition time of each image in the time series.
+
+        Returns
+        -------
+        go.Figure
+            A Plotly graph object interactive plot showing the acquisition time of each image in the time series.
+        """
+        df = self.dataframe()
+        df["y"] = 0
+
+        fig = px.line(
+            df,
+            x="time_start",
+            y="y",
+            hover_name="id",
+            markers=True,
+            labels={"time_start": ""},
+        )
+
+        fig.update_traces(
+            customdata=df[["id", "time_start"]],
+            hovertemplate="<b>%{customdata[0]}</b>"
+            + "<br>%{customdata[1]|%Y-%m-%d %H:%M:%S}",
+            line=dict(width=2, color="black"),
+            marker=dict(
+                size=12,
+                symbol="line-ns-open",
+                color="grey",
+                line=dict(width=1, color="black"),
+            ),
+        )
+
+        # Add circles for each image
+        fig.add_trace(
+            go.Scatter(
+                x=df.time_start,
+                y=df.y,
+                mode="markers",
+                hoverinfo="skip",
+                marker=dict(
+                    size=6,
+                    symbol="circle",
+                    color="white",
+                    line=dict(width=1, color="black"),
+                ),
+            )
+        )
+
+        fig.update_layout(
+            plot_bgcolor="white",
+            height=200,
+            hoverlabel=dict(bgcolor="white"),
+            showlegend=False,
+        )
+
+        fig.update_yaxes(visible=False)
+        fig.update_xaxes(linecolor="black", ticks="outside")
+
+        return fig
+
     def aggregate_time(
         self, frequency: str, reducer: Optional[Any] = None, keep_bandnames: bool = True
     ) -> "TimeSeries":
@@ -262,8 +352,7 @@ class TimeSeries(ee.imagecollection.ImageCollection):
             # Reducing makes images unbounded, so re-clip bounded images
             reduced = ee.Algorithms.If(geom.isUnbounded(), reduced, reduced.clip(geom))
 
-            if keep_bandnames:
-                reduced = ee.Image(reduced).rename(imgs.first().bandNames())
+            reduced = ee.Image(reduced).rename(imgs.first().bandNames())
 
             return ee.Algorithms.If(imgs.size().gt(0), reduced, None)
 
@@ -485,29 +574,32 @@ class TimeSeries(ee.imagecollection.ImageCollection):
 
     def interpolate_time(self, time: ee.Date, method: str = "linear") -> ee.Image:
         """Use interpolation to synthesize data at a given time within the time series. Based on the
-                interpolation method chosen, a certain number of images must be present in the time series
-                before and after the target date.
+        interpolation method chosen, a certain number of images must be present in the time series
+        before and after the target date.
 
-                Nearest and linear interpolation require 1 image before and after the selected time while
-                cubic interpolation requires 2 images before and after the selected time.
-        ClimatologyFrequencyEnum
-                Parameters
-                ----------
-                date : ee.Date
-                    The target date to interpolate data at. This must be within the time series period.
-                method : str, default linear
-                    The interpolation method to use, one of "nearest", "linear", or "cubic".
+        Nearest and linear interpolation require 1 image before and after the selected time while
+        cubic interpolation requires 2 images before and after the selected time.
 
-                Returns
-                -------
-                ee.Image
-                    Data interpolated to the target time from surrounding data in the time series.
+        Parameters
+        ----------
+        date : ee.Date
+            The target date to interpolate data at. This must be within the time series period.
+        method : str, default linear
+            The interpolation method to use, one of "nearest", "linear", or "cubic".
 
-                Example
-                -------
-                >>> ts = wxee.TimeSeries("IDAHO_EPSCOR/GRIDMET")
-                >>> target_date = ee.Date("2020-09-08T03")
-                >>> filled = ts.interpolate(target_date, "cubic")
+        Returns
+        -------
+        ee.Image
+            Data interpolated to the target time from surrounding data in the time series.
+
+        Examples
+        --------
+        >>> ts = wxee.TimeSeries("IDAHO_EPSCOR/GRIDMET")
+        >>> target_date = ee.Date("2020-09-08T03")
+
+        Interpolate weather data at the target date using cubic interpolation.
+
+        >>> filled = ts.interpolate(target_date, "cubic")
         """
         method_func = InterpolationMethodEnum.get_option(method)
 
@@ -593,7 +685,7 @@ class TimeSeries(ee.imagecollection.ImageCollection):
             minimum observations are not met, the primary image will be dropped. For example, a monthly time series
             reduced with a 10 day window and :code:`min_observations==3` would be empty because none of the
             windows would include enough observations.
-        reducer: Optional[ee.Reducer]
+        reducer : Optional[ee.Reducer]
             The reducer to apply to each rolling window. If none is given, ee.Reducer.mean will be used.
         keep_bandnames : bool, default True
             If true, the band names of the input images will be kept in the reduced images. If false, the name of the
@@ -610,19 +702,11 @@ class TimeSeries(ee.imagecollection.ImageCollection):
         >>> ts_smooth = ts.rolling_time(90, "day", "center", reducer=ee.Reducer.median())
         """
         reducer = ee.Reducer.mean() if not reducer else reducer
-        offset = 1 if align == "left" else 0.5 if align == "center" else 0
-        nudge = 1 if align == "left" else 0 if align == "center" else -1
 
         def roll_image(img: ee.Image) -> ee.Image:
             """Apply a rolling reducer to a single image using its temporal neighbors"""
             center = ee.Date(img.get("system:time_start"))
-            # The windows need to be nudged slightly to set the correct exclusive/inclusive order.
-            # For example, a left aligned window should exclude the left and include the right,
-            # and vice-versa for a right aligned window.
-            left = center.advance(window * offset * -1, unit).advance(nudge, "second")
-            right = left.advance(window, unit)
-
-            neighbors = self.filterDate(left, right)
+            neighbors = self._get_window(center, window, unit, align)
             smooth = neighbors.reduce(reducer)
 
             props = {
@@ -641,3 +725,93 @@ class TimeSeries(ee.imagecollection.ImageCollection):
             return ee.Algorithms.If(neighbors.size().lt(min_observations), None, smooth)
 
         return self.map(roll_image, opt_dropNulls=True)
+
+    def fill_gaps(
+        self,
+        window: int,
+        unit: str,
+        align: str = "center",
+        reducer: Optional[Any] = None,
+        fill_value: Optional[float] = None,
+    ) -> "TimeSeries":
+        """Apply gap-filling using a moving window reducer through time. Each image is unmasked using its reduced temporal neighbors.
+        If the window is not wide enough to include an unmasked value (e.g. if clouds occur in the same location in all images),
+        masked values will remain unless a :code:`fill_value` is specified.
+
+        Parameters
+        ----------
+        window : int
+            The number of time units to include in each rolling period.
+        unit : str
+            The time frequency of the window. One of "hour", "day", "week", "month", "year".
+        align : str, default "center"
+            The start location of the rolling window, relative to the primary image time. One of "left", "center",
+            "right". For example, a 3-day left-aligned window will include all images up to (but not including)
+            3 days prior to the primary image. Date ranges are exclusive in the alignment direction and inclusive
+            in the opposite direction, so each primary image will be included in its own window.
+        reducer : Optional[ee.Reducer]
+            The reducer to apply to each rolling window. If none is given, ee.Reducer.mean will be used.
+        fill_value : float
+            The value to fill any masked values with after applying initial gap-filling. If none is given, masked
+            values may remain if the window size is not large enough.
+
+        Returns
+        -------
+        wxee.time_series.TimeSeries
+            The time series with each image unmasked using its reduced neighbors.
+
+        Example
+        -------
+        >>> ts = wxee.TimeSeries("MODIS/006/MOD13A2)
+        >>> ts_filled = ts.fill_gaps(90, "day", "center", reducer=ee.Reducer.median())
+        """
+        reducer = ee.Reducer.mean() if not reducer else reducer
+
+        def fill_image(img: ee.Image) -> ee.Image:
+            """Unmask a single image by applying a rolling reducer to its temporal neighbors."""
+            center = ee.Date(img.get("system:time_start"))
+            neighbors = self._get_window(center, window, unit, align)
+            filler = neighbors.reduce(reducer)
+
+            filled = img.unmask(filler)
+            filled = filled.unmask(fill_value) if fill_value else filled
+
+            return filled
+
+        return self.map(fill_image)
+
+    def _get_window(
+        self, time: ee.Date, window: int, unit: str, align: str = "left"
+    ) -> "TimeSeries":
+        """Get all images within a window around a target date.
+
+        Parameters
+        ----------
+        time : ee.Date
+            The center date of the window.
+        window : int
+            The number of time units to include in the window.
+        unit : str
+            The time frequency of the window. One of "hour", "day", "week", "month", "year".
+        align : str, default "left"
+            The start location of the window, relative to the center time. One of "left", "center", "right". Date
+            ranges are exclusive in the alignment direction and inclusive in the opposite direction, so the center
+            date of the window is always included.
+
+        Returns
+        -------
+        wxee.time_series.TimeSeries
+            A time series containing all images in the window.
+        """
+        WindowAlignEnum.get_option(align)
+
+        offset = 1 if align == "left" else 0.5 if align == "center" else 0
+        nudge = 1 if align == "left" else 0 if align == "center" else -1
+
+        # The windows need to be nudged slightly to set the correct exclusive/inclusive order.
+        # For example, a left aligned window should exclude the left and include the right,
+        # and vice-versa for a right aligned window.
+        left = time.advance(window * offset * -1, unit).advance(nudge, "second")
+        right = left.advance(window, unit)
+
+        return self.filterDate(left, right)
